IF EXISTS (SELECT * FROM SYSOBJECTS O, SYSUSERS U
                    WHERE
	        O.NAME = 'CALCULA_PRESCRICAO'  AND
	        TYPE = 'P' 	                  AND
	        U.UID = O.UID                  AND
	        U.NAME = USER)
  DROP PROCEDURE CALCULA_PRESCRICAO
  GO

CREATE PROCEDURE CALCULA_PRESCRICAO
   @P_CDFILIAL     VARCHAR(04),
   @P_CDSERVFILI   VARCHAR(2000),
   @P_DTSERVINTE   DATETIME   ,
   @P_CDOPERADOR   VARCHAR(20),
   @P_CDANDAR      VARCHAR(2000),
   @P_CDALA        VARCHAR(2000),
   @P_CDLEITO      VARCHAR(2000),
   @P_ACRES_SOBR   VARCHAR(01),
   @P_CONFIG_ANTERIOR VARCHAR(01)
AS
   DECLARE @V_CDDIETA       VARCHAR(04);
   DECLARE @V_COUNTCARDAPIO NUMERIC;
   DECLARE @V_SERVPADRAO    VARCHAR(03);
   DECLARE @V_NRSEQUSERV    VARCHAR(02);
   DECLARE @V_IDOPCAO       VARCHAR(01);
   DECLARE @V_DESCCONF      VARCHAR(01);

   DECLARE @C_PRESCRICAO_CDPACIENTE  VARCHAR(12);
   DECLARE @C_PRESCRICAO_DTINTERNA   DATETIME;
   DECLARE @C_PRESCRICAO_CDSERPADDIE VARCHAR(03);
   DECLARE @C_PRESCRICAO_CDLEITO     VARCHAR(12);
   DECLARE @C_PRESCRICAO_DTPRESDIET  DATETIME;
   DECLARE @C_PRESCRICAO_CDDIETPAD   VARCHAR(04);
   DECLARE @C_PRESCRICAO_CDSERVFILI  VARCHAR(05);
   DECLARE @C_PRESCRICAO_CDDIETGRUP  VARCHAR(04);
   DECLARE @C_PRESCRICAO_IDFORMDIET  VARCHAR(01);
   DECLARE @C_PRESCRICAO_DSPRESDIET  VARCHAR(80);
   DECLARE @C_PRESCRICAO_IDTIPOSERV  VARCHAR(01);
   DECLARE @C_PRESCRICAO_PRES_CDDIETPAD VARCHAR(04);

   DECLARE @C_SERVINTE_CDPACIENTE  VARCHAR(12);
   DECLARE @C_SERVINTE_NRSEQUSERV  VARCHAR(02);
   DECLARE @C_SERVINTE_DTSERVINTE  DATETIME;

   DECLARE @C_COMPCARD_CDFILAUXPRAT VARCHAR(04);
   DECLARE @C_COMPCARD_CDPRATO      VARCHAR(10);
   DECLARE @C_COMPCARD_CDTIPOPRAT   VARCHAR(02);
   DECLARE @C_COMPCARD_NRORDPRATCAR NUMERIC;
   DECLARE @C_COMPCARD_IDOPCCNFMES  VARCHAR(01);

   DECLARE @V_CONSIDERA     VARCHAR(1);
   DECLARE @V_DTATUAL       DATETIME;
   DECLARE @V_CONST_N       VARCHAR(1);
   DECLARE @V_CONST_S       VARCHAR(1);
   DECLARE @V_UNDERLINE     VARCHAR(1);
   DECLARE @V_CONST_ZERO    VARCHAR(1);
   DECLARE @V_IDTPSERVHOSP_UM   VARCHAR(1);
   DECLARE @V_IDTPSERVHOSP_DOIS VARCHAR(1);
   DECLARE @V_HRINICSERV    VARCHAR(4);
   DECLARE @V_NUMBER_CONST_ZERO NUMERIC;
   DECLARE @V_IDESCOCONF VARCHAR(1);
   DECLARE @V_DSOBSCONF VARCHAR(250);
   DECLARE @V_USEDIETGROUP VARCHAR(1);

BEGIN
   SET @V_CONSIDERA = '0';
   SET @V_CONST_N   = 'N';
   SET @V_CONST_S   = 'S';
   SET @V_UNDERLINE = '_';
   SET @V_CONST_ZERO= '0';
   SET @V_IDTPSERVHOSP_UM   ='1';
   SET @V_IDTPSERVHOSP_DOIS ='2';
   SET @V_HRINICSERV = '2401';
   SET @V_NUMBER_CONST_ZERO = 0;
   SET @V_USEDIETGROUP ='N';

   SELECT @V_DTATUAL = CONVERT(DATETIME, CONVERT(VARCHAR, GETDATE(), 103), 103)
   FROM PARAMGERAL;
   IF (@P_DTSERVINTE > @V_DTATUAL) BEGIN
      SET @V_CONSIDERA = NULL;
   END;

   DECLARE C_PRESCRICAO CURSOR FOR
      SELECT INTERNAC.CDPACIENTE, INTERNAC.DTINTERNA , INTERNAC.CDSERPADDIE,
             INTERNAC.CDLEITO   , PRESDIET.DTPRESDIET, (CASE SERVICO.IDTIPOSERV WHEN @V_CONST_N THEN NULL ELSE PRESDIET.CDDIETPAD END) AS CDDIETPAD,
             DIETSERV.CDSERVFILI,
             ISNULL(DIETPADR.CDDIETGRUP, PRESDIET.CDDIETPAD) AS CDDIETGRUP, DIETPADR.IDFORMDIET,
             PRESDIET.DSPRESDIET, SERVICO.IDTIPOSERV, PRESDIET.CDDIETPAD AS PRES_CDDIETPAD
      FROM DIETSERV, SERVICO, DIETPADR, LEITO, INTERNAC, PRESDIET
      WHERE (PRESDIET.CDFILIAL    = INTERNAC.CDFILIAL)                                              AND
            (PRESDIET.CDPACIENTE  = INTERNAC.CDPACIENTE)                                            AND
            (PRESDIET.CDDIETPAD   = DIETSERV.CDDIETPAD)                                             AND
            (DIETSERV.CDFILIAL    = @P_CDFILIAL)                                                    AND
            (DIETSERV.CDFILIAL    = PRESDIET.CDFILIAL)                                              AND
            ((CHARINDEX(@V_UNDERLINE+DIETSERV.CDSERVFILI+@V_UNDERLINE, @P_CDSERVFILI) <> @V_NUMBER_CONST_ZERO) OR @P_CDSERVFILI IS NULL) AND
            ((CHARINDEX(@V_UNDERLINE+LEITO.CDALA        +@V_UNDERLINE, @P_CDALA)      <> @V_NUMBER_CONST_ZERO) OR @P_CDALA IS NULL)      AND
            ((CHARINDEX(@V_UNDERLINE+LEITO.CDANDAR      +@V_UNDERLINE, @P_CDANDAR)    <> @V_NUMBER_CONST_ZERO) OR @P_CDANDAR IS NULL)    AND
            ((CHARINDEX(@V_UNDERLINE+LEITO.CDLEITO      +@V_UNDERLINE, @P_CDLEITO)    <> @V_NUMBER_CONST_ZERO) OR @P_CDLEITO IS NULL )   AND
            (PRESDIET.DTPRESDIET  =
                   (SELECT MAX(DTPRESDIET)
                    FROM PRESDIET P
                    WHERE (P.CDFILIAL   = INTERNAC.CDFILIAL)   AND
                          (P.CDPACIENTE = INTERNAC.CDPACIENTE) AND
                          (CAST(CONVERT(VARCHAR, P.DTPRESDIET, 103) AS DATETIME) <= @P_DTSERVINTE))) AND

            ((ISNULL(SERVICO.HRINICSERV, @V_HRINICSERV) >= SUBSTRING(CONVERT(VARCHAR, GETDATE() , 108), 1, 2) + SUBSTRING(CONVERT(VARCHAR, GETDATE() , 108), 4, 2)) OR
             (@P_DTSERVINTE > CONVERT(DATETIME, GETDATE() , 103)))    AND

            (SERVICO.CDFILIAL     = DIETSERV.CDFILIAL)                                               AND
            (SERVICO.CDSERVFILI   = DIETSERV.CDSERVFILI)                                             AND
            (SERVICO.IDTPSERVHOSP IS NULL OR IDTPSERVHOSP NOT IN(@V_IDTPSERVHOSP_UM, @V_IDTPSERVHOSP_DOIS))                          AND
            (INTERNAC.DTALTA IS NULL)                                                                AND
            (LEITO.CDFILIAL = INTERNAC.CDFILIAL)                                                     AND
            (LEITO.CDLEITO  = INTERNAC.CDLEITO)                                                      AND
            (DIETPADR.CDDIETPAD   = PRESDIET.CDDIETPAD);

   OPEN C_PRESCRICAO;

   FETCH NEXT FROM C_PRESCRICAO INTO @C_PRESCRICAO_CDPACIENTE,  @C_PRESCRICAO_DTINTERNA,
                                     @C_PRESCRICAO_CDSERPADDIE, @C_PRESCRICAO_CDLEITO,
                                     @C_PRESCRICAO_DTPRESDIET,  @C_PRESCRICAO_CDDIETPAD,
                                     @C_PRESCRICAO_CDSERVFILI,  @C_PRESCRICAO_CDDIETGRUP,
                                     @C_PRESCRICAO_IDFORMDIET,  @C_PRESCRICAO_DSPRESDIET,
                                     @C_PRESCRICAO_IDTIPOSERV,  @C_PRESCRICAO_PRES_CDDIETPAD;
   WHILE (@@FETCH_STATUS = @V_NUMBER_CONST_ZERO) BEGIN
      SELECT @V_SERVPADRAO = MAX(CDSERPADDIE)
      FROM SERVICO
      WHERE (CDFILIAL   = @P_CDFILIAL) AND
            (CDSERVFILI = @C_PRESCRICAO_CDSERVFILI);

      IF ((@V_SERVPADRAO IS NULL) OR (@V_SERVPADRAO = @C_PRESCRICAO_CDSERPADDIE)) BEGIN
         SELECT @V_COUNTCARDAPIO = COUNT(*)
         FROM CARDADIA, COMPCARD
         WHERE (CARDADIA.CDFILIAL   = @P_CDFILIAL)              AND
               (CARDADIA.CDSERVFILI = @C_PRESCRICAO_CDSERVFILI) AND
               (CARDADIA.CDDIETPAD  = @C_PRESCRICAO_CDDIETPAD)  AND
               (CARDADIA.DTCARDAPIO = @P_DTSERVINTE)            AND
               (COMPCARD.CDFILIAL   = CARDADIA.CDFILIAL)        AND
               (COMPCARD.NRCARDAPIO = CARDADIA.NRCARDAPIO);

         SET @V_CDDIETA = @C_PRESCRICAO_CDDIETPAD;

         IF (@V_COUNTCARDAPIO = @V_NUMBER_CONST_ZERO) BEGIN
            IF (@C_PRESCRICAO_IDTIPOSERV = @V_CONST_N) BEGIN
              SET @V_CDDIETA = NULL;
              SET @V_USEDIETGROUP = @V_CONST_N;
            END ELSE BEGIN
              SET @V_CDDIETA = @C_PRESCRICAO_CDDIETGRUP;
              SET @V_USEDIETGROUP = @V_CONST_S;
            END;
         END;

         SELECT @V_NRSEQUSERV = RIGHT(REPLICATE(@V_CONST_ZERO,2) + CONVERT(VARCHAR, (ISNULL(MAX(NRSEQUSERV), 0) + 1)), 2)
         FROM SERVINTE
         WHERE (CDFILIAL   = @P_CDFILIAL)              AND
               (CDPACIENTE = @C_PRESCRICAO_CDPACIENTE) AND
               (DTSERVINTE = @P_DTSERVINTE);

         SELECT @V_COUNTCARDAPIO = COUNT(*)
         FROM CARDADIA, COMPCARD
         WHERE (CARDADIA.CDFILIAL   = @P_CDFILIAL)              AND
               (CARDADIA.CDSERVFILI = @C_PRESCRICAO_CDSERVFILI) AND
               (CARDADIA.CDDIETPAD  = @V_CDDIETA OR @V_CDDIETA IS NULL)AND
               (CARDADIA.DTCARDAPIO = @P_DTSERVINTE)            AND
               (COMPCARD.CDFILIAL   = CARDADIA.CDFILIAL)        AND
               (COMPCARD.NRCARDAPIO = CARDADIA.NRCARDAPIO);
         IF (@C_PRESCRICAO_IDFORMDIET = @V_CONST_ZERO) AND (@V_COUNTCARDAPIO <> @V_NUMBER_CONST_ZERO) BEGIN
            IF (@C_PRESCRICAO_IDTIPOSERV = @V_CONST_N) BEGIN
              SET @V_CDDIETA = @C_PRESCRICAO_PRES_CDDIETPAD;
            END ELSE BEGIN
              SET @V_CDDIETA = @C_PRESCRICAO_CDDIETPAD;
            END;

            IF (@V_USEDIETGROUP = @V_CONST_S) BEGIN
                SET @V_CDDIETA = @C_PRESCRICAO_CDDIETGRUP;
            END;

            INSERT INTO SERVINTE
               (CDPACIENTE,  DTSERVINTE, NRSEQUSERV, DTINTERNA,
                CDFILIAL,   CDSERVFILI, CDDIETPAD, STPACIACOM,
                IDIMPRSERV,  CDNUTRICIO, CDDIETGRUPSI, DSSERVINTE)
            VALUES
               (@C_PRESCRICAO_CDPACIENTE ,   @P_DTSERVINTE, @V_NRSEQUSERV,
                @C_PRESCRICAO_DTINTERNA  ,   @P_CDFILIAL  , @C_PRESCRICAO_CDSERVFILI,
                @V_CDDIETA, @V_CONST_ZERO, @V_CONST_S, @P_CDOPERADOR,
                @C_PRESCRICAO_CDDIETGRUP, @C_PRESCRICAO_DSPRESDIET);

            DECLARE C_COMPCARD CURSOR FOR
               SELECT COMPCARD.CDFILAUXPRAT, COMPCARD.CDPRATO,
                      COMPCARD.CDTIPOPRAT,   COMPCARD.NRORDPRATCAR,
                      CNFCAMES.IDOPCCNFMES
               FROM CNFCAMES, CARDADIA, COMPCARD
               WHERE (CARDADIA.CDFILIAL     = @P_CDFILIAL)              AND
                     (CARDADIA.CDSERVFILI   = @C_PRESCRICAO_CDSERVFILI) AND
                     (CARDADIA.CDDIETPAD    = @V_CDDIETA OR CARDADIA.CDDIETPAD IS NULL) AND
                     (CARDADIA.DTCARDAPIO   = @P_DTSERVINTE)            AND
                     (COMPCARD.CDFILIAL     = CARDADIA.CDFILIAL)        AND
                     (COMPCARD.NRCARDAPIO   = CARDADIA.NRCARDAPIO)      AND
                     (CNFCAMES.DTREFECNFMES = CARDADIA.DTMESCARD)       AND
                     (CNFCAMES.CDSERVFILI   = CARDADIA.CDSERVFILI)      AND
                     (CNFCAMES.CDFILIAL     = CARDADIA.CDFILIAL)        AND
                     (CNFCAMES.NRSEQCNFMES  = COMPCARD.NRSEQPRATCAR);

            OPEN C_COMPCARD;

            FETCH NEXT FROM C_COMPCARD INTO @C_COMPCARD_CDFILAUXPRAT, @C_COMPCARD_CDPRATO,
                                            @C_COMPCARD_CDTIPOPRAT,  @C_COMPCARD_NRORDPRATCAR,
                                            @C_COMPCARD_IDOPCCNFMES;

            WHILE (@@FETCH_STATUS = @V_NUMBER_CONST_ZERO) BEGIN

               IF (@C_COMPCARD_IDOPCCNFMES = @V_CONST_S) BEGIN
                  SET @V_DESCCONF = @V_CONST_N;
               END
               ELSE BEGIN
                  SET @V_DESCCONF = @V_CONST_S;
               END;

               IF (@P_CONFIG_ANTERIOR = @V_CONST_S) BEGIN
                      SELECT
                         @V_IDESCOCONF = IDESCOCONF, @V_DSOBSCONF = DSOBSCONF
                      FROM
                         CONFESCO CF
                      WHERE
                         (CF.CDFILIAL = @P_CDFILIAL)                 AND
                         (CF.CDPACIENTE = @C_PRESCRICAO_CDPACIENTE)  AND
                         (CF.DTSERVINTE = @P_DTSERVINTE)             AND
                         (CF.CDTIPOPRAT = @C_COMPCARD_CDTIPOPRAT)    AND
                         (CF.CDSEQUCOPR = RIGHT(REPLICATE(@V_CONST_ZERO,4) + CONVERT(VARCHAR, @C_COMPCARD_NRORDPRATCAR), 4)) AND
                         (CF.NRSEQUSERV = (SELECT MAX(CONVERT(INT,SERVINTE.NRSEQUSERV))
                                           FROM SERVINTE
                                           WHERE
                                             (SERVINTE.CDFILIAL = CF.CDFILIAL    ) AND
                                             (SERVINTE.DTSERVINTE = CF.DTSERVINTE) AND
                                             (SERVINTE.CDPACIENTE = CF.CDPACIENTE) AND
                                             (CONVERT(INT,SERVINTE.NRSEQUSERV)<(CONVERT(INT,@V_NRSEQUSERV))) AND
                                             (SERVINTE.CDSERVFILI = @C_PRESCRICAO_CDSERVFILI)
                                          ));

                         IF (@V_IDESCOCONF IS NULL) BEGIN
                             SET @V_IDESCOCONF = @V_DESCCONF;
                         END;
               END;


               INSERT INTO CONFESCO
                  (CDFILIAL, CDPACIENTE, DTSERVINTE, NRSEQUSERV, CDSEQUCOPR, IDOPCAOCONF,
                   IDESCOCONF, CDFILAUXPRAT, CDPRATO, CDTIPOPRAT, DSOBSCONF)
               VALUES
                  (@P_CDFILIAL, @C_PRESCRICAO_CDPACIENTE, @P_DTSERVINTE, @V_NRSEQUSERV,
                   RIGHT(REPLICATE(@V_CONST_ZERO,4) + CONVERT(VARCHAR, @C_COMPCARD_NRORDPRATCAR), 4),
                   @C_COMPCARD_IDOPCCNFMES, (CASE @P_CONFIG_ANTERIOR WHEN @V_CONST_S THEN @V_IDESCOCONF ELSE @V_DESCCONF END), @C_COMPCARD_CDFILAUXPRAT,
                   @C_COMPCARD_CDPRATO, @C_COMPCARD_CDTIPOPRAT,
                   (CASE @P_CONFIG_ANTERIOR WHEN @V_CONST_S THEN @V_DSOBSCONF ELSE NULL END));

               SET @V_IDESCOCONF = NULL;
               SET @V_DSOBSCONF  = NULL;

            FETCH NEXT FROM C_COMPCARD INTO @C_COMPCARD_CDFILAUXPRAT, @C_COMPCARD_CDPRATO,
                                            @C_COMPCARD_CDTIPOPRAT,   @C_COMPCARD_NRORDPRATCAR,
                                            @C_COMPCARD_IDOPCCNFMES;

            END;

           CLOSE C_COMPCARD;
           DEALLOCATE C_COMPCARD;
         END;
      END;

      /*DELETE TODAS AS DIETAS ORAIS DO PACIENTE SE A OPO SOBREPOR FOI MARCADA (menos a que foi includa agora)*/
      IF (@P_ACRES_SOBR = @V_CONST_S) BEGIN

        DELETE FROM CONFESCO
        WHERE
           (CONFESCO.CDFILIAL = @P_CDFILIAL) AND
           (CONFESCO.CDPACIENTE = @C_PRESCRICAO_CDPACIENTE) AND
           (CONFESCO.DTSERVINTE = @P_DTSERVINTE)            AND
           (CONFESCO.NRSEQUSERV IN (SELECT SERVINTE.NRSEQUSERV
                                    FROM SERVINTE
                                    WHERE
                                     (SERVINTE.CDPACIENTE = @C_PRESCRICAO_CDPACIENTE) AND
                                     (SERVINTE.CDFILIAL = @P_CDFILIAL)                AND
                                     (SERVINTE.STPACIACOM = @V_CONST_ZERO)            AND
                                     (SERVINTE.IDIMPRSERV = @V_CONST_S)               AND
                                     (SERVINTE.DTSERVINTE = @P_DTSERVINTE)            AND
                                     (SERVINTE.NRSEQUSERV <> @V_NRSEQUSERV)           AND
                                     (SERVINTE.CDSERVFILI = @C_PRESCRICAO_CDSERVFILI)));
        DELETE FROM SERVINTE
        WHERE
         (SERVINTE.CDPACIENTE = @C_PRESCRICAO_CDPACIENTE) AND
         (SERVINTE.CDFILIAL = @P_CDFILIAL)                AND
         (SERVINTE.STPACIACOM = @V_CONST_ZERO)            AND
         (SERVINTE.IDIMPRSERV = @V_CONST_S)               AND
         (SERVINTE.DTSERVINTE = @P_DTSERVINTE)            AND
         (SERVINTE.NRSEQUSERV <> @V_NRSEQUSERV) AND
         (SERVINTE.CDSERVFILI = @C_PRESCRICAO_CDSERVFILI);
      END;

      FETCH NEXT FROM C_PRESCRICAO INTO @C_PRESCRICAO_CDPACIENTE,  @C_PRESCRICAO_DTINTERNA,
                                        @C_PRESCRICAO_CDSERPADDIE, @C_PRESCRICAO_CDLEITO,
                                        @C_PRESCRICAO_DTPRESDIET,  @C_PRESCRICAO_CDDIETPAD,
                                        @C_PRESCRICAO_CDSERVFILI,  @C_PRESCRICAO_CDDIETGRUP,
                                        @C_PRESCRICAO_IDFORMDIET,  @C_PRESCRICAO_DSPRESDIET,
                                        @C_PRESCRICAO_IDTIPOSERV,  @C_PRESCRICAO_PRES_CDDIETPAD;
   END;

   CLOSE C_PRESCRICAO;
   DEALLOCATE C_PRESCRICAO;
END;
GO


-- VERSAO --
EXEC ATUVEROBJ 'CALCULA_PRESCRICAO','4.11.034';
GO

